// EODetailDatabaseDataSource.h
// Copyright (c) 1994, NeXT Computer, Inc. All rights reserved.

#import	"EODataSources.h"

#import <foundation/NSArray.h>
#import <foundation/NSData.h>
#import <foundation/NSDictionary.h>
#import <foundation/NSObject.h>
#import <foundation/NSString.h>

@class EODatabaseChannel;
@class EOQualifier;
@class EOEntity;
@class EODatabaseDataSource;

// EODetailDBDataSource is a detail data source which has its master
// handle all the fetching. These are used to implement master-detail
// browsers.
//
// Instances of this class are created by sending
// -dataSourceQualifiedByKey:ofObject: to an EODatabaseDataSource or
// EODetailDBDataSource.  You should never allocate and initialize an
// EODetailDBDataSource yourself.


@interface EODetailDatabaseDataSource:NSObject <EOMasterDataSources, EOQualifiableDataSources, EORollbackDataSources>
{
    NSString *detailKey;
    EODatabaseDataSource *masterDataSource;
    NSArray *dbArray;
    id masterObject;
    EOEntity *entity;
    BOOL isRelationshipToOne;
}

- initWithMasterDataSource: (EODatabaseDataSource *)m 
    entity: (EOEntity *)e;
    // Initializes a newly allocated EODetailDBDataSource with the given
    // information.  Instances of this class are created by sending
    // -dataSourceQualifiedByKey: to an EODatabaseDataSource or
    // EODetailDBDataSource.  You should never allocate and initialize an
    // EODetailDBDataSource yourself.

- (EOEntity *)entity;
    // Returns the entity that the receiver fetches object for.

- (EODatabaseDataSource *)masterDataSource;
    // Returns the data source that the receiver is a detail data source for.

- masterObject;
    // Returns the object in the master data source that roots the detail data
    // source.

- (NSString *)detailKey;
    // Returns the key used to fetch objects.  This is a relationship key from
    // the master object.

- (NSArray *)array;
    // Returns the array of objects supplied by the receiver.

@end
