// OracleAdaptor.h
// Copyright (c) 1994, NeXT Computer, Inc.  All rights reserved.

#import <eoaccess/eoaccess.h>

@class OracleContext;

// These keys define the standard connection information for an Oracle logon.
//
// If there is no value for the HostMachineKey then the a connection string
// of the form "userName/password@serverId" is generated.
//
// If all the values except the one for serverId are absent, then the
// connection string will just be the value for serverId.

#define ServerIdKey @"serverId"
#define HostMachineKey @"hostMachine"
#define UserNameKey @"userName"
#define PasswordKey @"password"

// If this key is present in the connection dictionary, the other keys are
// ignored and this string is passed returned from the -oracleConnectionString
// method.

#define ConnectionStringKey @"connectionString"

// If this key is present, it will be used as the setting to NLS_LANG
// used to specify the language and character set for server connections.
// On J systems this option defaults to japanese_japan.jeuc

#define NlsLangKey @"NLS_LANG"


@interface OracleAdaptor:EOAdaptor
{
    unsigned short _connectedLogons;
}

- initWithName:(NSString *)name;
    // Designated initializer

- (Class)expressionClass;
- (Class)adaptorContextClass;
- (Class)adaptorChannelClass;
    // The classes for the various pieces of the adaptor.

- createAdaptorContext;

- (NSString *)oracleConnectionString;
    // This returns the user name, password, host machine, and server id as a
    // string suitable to be fed to orlon().

- formatAttribute:(EOAttribute *)attribute;
- formatValue:value forAttribute:(EOAttribute *)attribute;
- (BOOL)isValidQualifierType:(NSString *)typeName;
- (BOOL)hasValidConnectionDictionary;
- (BOOL)hasOpenChannels;

- (void)oracleContextWillConnect:(OracleContext *)logon;
    // This is called by an OracleContext when it will try to connect using
    // the adaptor's connection information.

- (void)oracleContextDidDisconnect:(OracleContext *)logon;
    // This is called by an OracleContext which has just disconnected.

- (NSArray *)connectionKeys;
    // A hook to allow programs to prompt the user for a connection dictionary.

- (void)prepareEnvironmentForConnect;
- (void)resetEnvironmentAfterConnect;
    // These should bracket all calls to orlon() to set
    // the NLS_LANG environment variable setting to the
    // value specified in the model connection dictionary.
@end

