// OracleChannel.h 
// Copyright (c) 1994, NeXT Computer, Inc.  All rights reserved.

#import "ociapr.h"
#import <eoaccess/eoaccess.h>

@class OracleContext;

@interface OracleChannel:EOAdaptorChannel
{
    Cda_Def _cda;
    OracleContext *_context;
    
    EOAttribute *_longAttribute;
    id _longValue;
    id _currentExpression;
    
    NSArray *_selectedAttributes;
    NSMutableArray *_selectedColumns;
    unsigned _fetchBufferLength;
    unsigned _rowCapacity;
    unsigned _rowsInBuffer;
    unsigned _nextRowInBuffer;
    unsigned _rowsProcessedCount;
    
    BOOL _fetchInProgress;
    BOOL _opened;
}

- initWithOracleContext:(OracleContext *)logon;

- (Cda_Def *)cursorDataArea;
    // This returns NULL if the channel is not connected.

- (void)setFetchBufferLength:(unsigned)length;
- (unsigned)fetchBufferLength;
    // These are to manipulate how much memory to use as a fetch buffer.  The
    // larger the buffer, the more rows that can be returned for each round
    // trip to the server.

- (void)setDebugEnabled:(BOOL)yn;
- (BOOL)isDebugEnabled;
    // These control debugging output.  They are for debugging only -- do not
    // depend on the contents of the output (if any) in a deployed app.

@end


@interface NSObject (OracleChannelDelegation)

- (BOOL)oracleChannel:(OracleChannel *)channel
    willReportDatabaseError:(NSString *)error;
    // This delegate message is called whenever the channel encounters an
    // error reported by the Oracle server.  The cda, lda, and hda can be
    // gotten from the channel and context to determine exactly what error has
    // occurred.  The string error is the text of the error message which will
    // be sent to the adaptor's reportError: method.  The delegate can return
    // NO to prevent the channel from calling reportError:.

@end

